@@ -67,12 +67,6 @@ class VersionError(Exception):
 class NoDefault:
     pass
 
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set, ImmutableSet
-    set, frozenset = Set, ImmutableSet
-
 import re, fnmatch
 import operator
 import threading
@@ -420,6 +414,30 @@ class SQLObjectTable(Table):
     def _getattrFromUnicodeColumn(self, column, attr):
         return self.UnicodeFieldClass(self.tableName, column.dbName, attr, column)
 
+class SQLObjectTableWithJoins(SQLObjectTable):
+
+    def __getattr__(self, attr):
+        if attr+'ID' in [k for (k,v) in self.soClass.sqlmeta.columns.items() if v.foreignKey]:
+            column = self.soClass.sqlmeta.columns[attr+'ID']
+            return self._getattrFromForeignKey(column, attr)
+        elif attr in [x.joinMethodName for x in self.soClass.sqlmeta.joins]:
+            join = [x for x in self.soClass.sqlmeta.joins if x.joinMethodName == attr][0]
+            return self._getattrFromJoin(join, attr)
+        else:
+            return SQLObjectTable.__getattr__(self, attr)
+
+    def _getattrFromForeignKey(self, column, attr):
+        ret =  getattr(self, column.name) == \
+              getattr(self.soClass, '_SO_class_'+column.foreignKey).q.id
+        return ret
+
+    def _getattrFromJoin(self, join, attr):
+        if hasattr(join, 'otherColumn'):
+            return AND(join.otherClass.q.id == Field(join.intermediateTable, join.otherColumn),
+                            Field(join.intermediateTable, join.joinColumn) == self.soClass.q.id)
+        else:
+            return getattr(join.otherClass.q, join.joinColumn)==self.soClass.q.id
+
 class TableSpace:
     TableClass = Table
 
@@ -835,13 +853,13 @@ def NOTIN(item, list):
         return NOT(_IN(item, list))
 
 def STARTSWITH(expr, pattern):
-    return LIKE(expr, _LikeQuoted(pattern) + '%', escape='\\')
+    return SQLOp("LIKE", expr, _LikeQuoted(pattern) + '%')
 
 def ENDSWITH(expr, pattern):
-    return LIKE(expr, '%' + _LikeQuoted(pattern), escape='\\')
+    return SQLOp("LIKE", expr, '%' + _LikeQuoted(pattern))
 
 def CONTAINSSTRING(expr, pattern):
-    return LIKE(expr, '%' + _LikeQuoted(pattern) + '%', escape='\\')
+    return SQLOp("LIKE", expr, '%' + _LikeQuoted(pattern) + '%')
 
 def ISNULL(expr):
     return SQLOp("IS", expr, None)
@@ -883,7 +901,7 @@ class _LikeQuoted:
             values = []
             if self.prefix:
                 values.append("'%s'" % self.prefix)
-            s = _quote_like_special(sqlrepr(s, db), db)
+            s = _quote_percent(sqlrepr(s, db), db)
             values.append(s)
             if self.postfix:
                 values.append("'%s'" % self.postfix)
@@ -892,17 +910,16 @@ class _LikeQuoted:
             else:
                 return " || ".join(values)
         elif isinstance(s, basestring):
-            s = _quote_like_special(sqlrepr(s, db)[1:-1], db)
+            s = _quote_percent(sqlrepr(s, db)[1:-1], db)
             return "'%s%s%s'" % (self.prefix, s, self.postfix)
         else:
            raise TypeError, "expected str, unicode or SQLExpression, got %s" % type(s)
 
-def _quote_like_special(s, db):
-    if db == 'postgres':
-        escape = r'\\'
+def _quote_percent(s, db):
+    if db in ('postgres', 'mysql'):
+        s = s.replace('%', '\\%')
     else:
-        escape = '\\'
-    s = s.replace('\\', r'\\').replace('%', escape+'%').replace('_', escape+'_')
+        s = s.replace('%', '%%')
     return s
 
 ########################################
@@ -1131,17 +1148,11 @@ class Outer:
 class LIKE(SQLExpression):
     op = "LIKE"
 
-    def __init__(self, expr, string, escape=None):
+    def __init__(self, expr, string):
         self.expr = expr
         self.string = string
-        self.escape = escape
     def __sqlrepr__(self, db):
-        escape = self.escape
-        like = "%s %s (%s)" % (sqlrepr(self.expr, db), self.op, sqlrepr(self.string, db))
-        if escape is None:
-            return "(%s)" % like
-        else:
-            return "(%s ESCAPE %s)" % (like, sqlrepr(escape, db))
+        return "(%s %s (%s))" % (sqlrepr(self.expr, db), self.op, sqlrepr(self.string, db))
     def components(self):
         return [self.expr, self.string]
     def execute(self, executor):
@@ -1159,16 +1170,15 @@ class LIKE(SQLExpression):
 class RLIKE(LIKE):
     op = "RLIKE"
 
-    op_db = {
-        'firebird': 'RLIKE',
-        'maxdb': 'RLIKE',
-        'mysql': 'RLIKE',
-        'postgres': '~',
-        'sqlite': 'REGEXP'
-    }
-
     def _get_op(self, db):
-        return self.op_db.get(db, 'LIKE')
+        if db in ('mysql', 'maxdb', 'firebird'):
+            return "RLIKE"
+        elif db == 'sqlite':
+            return "REGEXP"
+        elif db == 'postgres':
+            return "~"
+        else:
+            return "LIKE"
     def __sqlrepr__(self, db):
         return "(%s %s (%s))" % (
             sqlrepr(self.expr, db), self._get_op(db), sqlrepr(self.string, db)
